import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * opm2mml
 * A .opm to MXDRV .mml converter
 * @author eXOdus (Nelson A.)
 * Finished 1/3/2016
 */

public class opm2mml 
{
	public static void main(String[] args) 
	{
		credits();
		if(args.length == 0) usage();
		else {
			String fileName = args[0];
			String line = null;
			ArrayList<String[]> opms = new ArrayList<String[]>();
			ArrayList<String[]> mml = new ArrayList<String[]>();
			try {
				FileReader fileReader = new FileReader(fileName);
				BufferedReader bufferedReader = new BufferedReader(fileReader);
				int i = 0; //instrument counter
				while ((line = bufferedReader.readLine()) != null) {
					if (!(line.startsWith("//"))) {
						if (line.startsWith("@:")) {
							opms.add(new String[5]);
							int j = 0;
							line = bufferedReader.readLine();
							while (line != null && line.length() != 0) {
								if (!line.startsWith("LFO:")) {
									opms.get(i)[j] = line;
									line = bufferedReader.readLine();
									if (j < 5) {
										j++;
									}
								} else
									line = bufferedReader.readLine();
							}
							i++;
						}
					}
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				System.out.println("Can't find file '" + fileName + "'!");
			}
			for (int i = 0; i < opms.size(); i++) {
				mml.add(toMML(opms.get(i)));
			}
			String newFileName = fileName.substring(0, fileName.lastIndexOf('.')) + ".mml";
			try {
				FileWriter fileWriter = new FileWriter(newFileName);

				BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

				bufferedWriter.write("/* MXDRV MML patches generated by opm2mml */ \n /* eXOdus, 2016 */ \n \n");
				System.out.print("[");

				for (int i = 0; i < mml.size(); i++) {
					bufferedWriter.write("@" + i + "=" + "{ \n " + "\t /* AR DR SR RR SL OL KS ML DT1 DT2 AME */ \n");
					for (int j = 0; j < mml.get(i).length; j++) {
						if (j != 4)
							bufferedWriter.write("\t" + mml.get(i)[j] + "\n");
						else
							bufferedWriter.write("\t /* CON FL OP */ \n \t" + mml.get(i)[j] + "\n");

					}
					bufferedWriter.write("} \n \n");
					System.out.print("=");
				}
				System.out.print("]");

			} catch (IOException e) {
				System.out.println("Can't write to file " + newFileName + "!");
			}
			System.out.println("\n \n OPM patches successfully converted and written to " + newFileName);
		}
	}
	
	private static void credits()
	{
		System.out.println("opm2mml by eXOdus (Nelson Addison)");
		System.out.println("A tool to make life easier for MXDRV MML users");
		System.out.println("Converts a valid opm file to MXDRV MML \n");
	}
	
	private static String[] toMML(String[] opm)
	{
		int[][] opmValues = getParams(opm);
		String[] mml = new String[5];
		int[][] values = new int[5][];
		for(int i = 0; i < values.length; i++)
		{
			mml[i] = "";
			if(i == 4) values[i] = new int[3];
			else values[i] = new int[11];
		}
		
		values[4][0] = opmValues[0][2];
		values[4][1] = opmValues[0][1];
		values[4][2] = 15;
		
		for(int i = 0; i < values.length - 1; i++)
		{
			for(int j = 0; j < values[i].length; j++)
			{
				values[i][j] = opmValues[i+1][j];
			}
		}
		
		for(int i = 0; i < mml.length; i++)
		{
			for(int j = 0; j < values[i].length; j++)
			{
				if(i == 4 && j == 2) mml[i] += values[i][j];
				else mml[i] += values[i][j] + ", ";
			}
		}
 		return mml;
	}
	
	private static int[][] getParams(String[] opm)
	{
		int[][] values = new int[opm.length][];
		for(int i = 0; i < opm.length; i++)
		{
			if(i == 0) values[i] = new int[7];
			else values[i] = new int[11];
		}
		
		for(int i = 0; i < opm.length; i++)
		{
			int j = 3;
			int curValue = 0;
			while(j < opm[i].length() - 1)
			{
				if( Character.isDigit(opm[i].charAt(j)) && curValue < values[i].length)
				{
					int startPos = j;
					int endPos = j+1;
					while(Character.isDigit(opm[i].charAt(j)))
					{
						endPos = j + 1;
						j++;
					}
					values[i][curValue] = new Scanner(opm[i].substring(startPos, endPos)).nextInt();
					curValue++;
				}
				j++;
			}
		}
		return values;
	}
	
	private static void usage()
	{
		System.out.println("Usage: opm2mml <infile> [outfile]");
	}
}
